#setwd("C:/")  #insert path to where data and code for functions are kept

load("XX.Rdata")     #insert file name  ## contains objects: diameters, increment, plotnames, spgroup, surviv, yrvec, mtree, covars

ierr <- .1           #width (in cm) of error window for increments

source('diametergrow_functions.r')  #initialize and functions

COVARIATES <- T     # T if there are covariates
ncovars    <- 3     # insert number of covariates in 'covars'


#priors
prior.mu  <- .3      #prior mean variance for mean growth rate
prior.Vmu <- 10

#covariates
if(COVARIATES){


 covars <- as.matrix(covars)
  intercept <- rep(1,nrow(covars))
 
 covars <- cbind(intercept,covars)

  xnames <- c('intercept','C.St2','dbh','elev')  #covariate names to use - change to whatever applicable
  
  xx <- covars[,xnames]
  X  <- xx[aincr[,1],]
  
  ncovars   <- ncol(X) - 1
  nx        <- nrow(X)

  prior.mu  <- rep(0,(1+ncovars))
  prior.IVm <- solve(diag(prior.mu+20))

  tmat  <- matrix(0,nrow(X),(nt-1))               #matrix for year effects
  tvector  <- matrix(c(1:(nt-1)),nrow(dgrow),(nt-1),byrow=T)[aincr]
  tind     <- cbind(c(1:nx),tvector)
  tmat[tind] <- tvector


}

nyr <- length(yrvec)

mui  <- .4           #individual effects var
vi1  <- 2
vi2  <- mui*(vi1 - 1)

mup  <- .4           #plot effects var
pi1  <- 2
pi2  <- mup*(pi1 - 1)

mus  <- .1^2            #process error
s1   <- 10
s2   <- mus*(s1 - 1)

muw  <- .2^2            #diameter error
w1   <- ndobs*2
w2   <- muw*(w1 - 1)

muv  <- .02^2        #increment error
v11  <- niobs
v22  <- muv*(v11 - 1)
  if(niobs == 0){v11 <- 0; v22 <- 0}

# initial values
mu      <- tnorm(1,0,1,prior.mu,1)
sig     <- 1/rgamma(1,s1,s2)
sigd    <- 1/rgamma(1,vi1,vi2)
sigp    <- 1/rgamma(1,pi1,pi2)
w.error <- 1/rgamma(1,w1,w2)
v.error <- 1/rgamma(1,vi1,vi2)
beta.i  <- rep(0,n)             #individual random effects
beta.p  <- rep(0,mplot)         #plot random effects
beta.t  <- rep(0,(nt-1))        #fixed year effects
 peffect <- matrix( rep(beta.p,times=(nm*(nt-1))),nrow=n,byrow=T)
  mumat   <- peffect*0
  teffect <- matrix(rep(beta.t,each=n),nrow=n,byrow=F)
  
ng <- 50000
burnin <- 10000
printseq <- seq(10,ng,by=1000)

if(COVARIATES){
  agibbs <- matrix(NA,ng,(ncovars+1))
  colnames(agibbs) <- xnames#paste('a',c(1:(ncovars+1)),sep='-')
}
mgibbs  <- rep(0,ng)
bigibbs <- rep(0,n)
bpgibbs <- rep(0,mplot)
bp2gibbs <- rep(0,mplot)
ingibbs <- matrix(0,ng,4)  #a sample of individuals
  ins  <- sort(sample(c(1:n),4))
sgibbs  <- matrix(0,ng,5)
  colnames(sgibbs) <- c('proc','ind','plot','diam','incr')
tgibbs  <- matrix(0,ng,(nt-1)); colnames(tgibbs) <- yrvec[-nt]
dgibbs  <- diam.t*0         #sums and sums of squares, diam
d2gibbs <- diam.t*0
ggibbs  <- diam.t[,-nt]*0   #sums and sums of squares, growth
g2gibbs <- diam.t[,-nt]*0
ad <- rep(0,ng)            #acceptance counter by step
ai <- rep(0,n)             #acceptance counter by tree


#log scale
ldgibbs  <- diam.t*0         #sums and sums of squares, diam
ld2gibbs <- diam.t*0
lggibbs  <- diam.t[,-nt]*0   #sums and sums of squares, growth
lg2gibbs <- diam.t[,-nt]*0

printseq <- seq(10,ng,by=100)

for(g in 1:ng){

  ftmp <- f.update()
    mu      <- ftmp$mu
    beta.t  <- ftmp$beta.t
    alpha   <- ftmp$alpha

    muVec   <- ftmp$muVec
    betaMat <- ftmp$betaMat

    teffect <- betaMat[nindex,]

    mumat   <- mu
    if(length(mu) == 1)mumat <- matrix(mu,n,(nt-1))
    if(!COVARIATES)mumat <- matrix(muVec[nindex],n,(nt-1))

  beta.i <- in.update()
    ieffect <- matrix(rep(beta.i,each=(nt-1)),nrow=n,byrow=T)

  if(mplot > 1) {
     beta.p <- p.update()
     sigp <- sp.update()  #plots
  }

  if(mplot == 1){
    beta.p <- 0
    sigp   <- 0
  }

  peffect <- matrix( rep(beta.p,times=(nm*(nt-1))),nrow=n,byrow=T)
  peffect <- peffect*0  #setting plot effect to zero
  sig  <- se.update()  #proc
  sigd <- sd.update()  #individuals

  dtmp <- di.update()
     diam.t  <- dtmp$diam.t
     w.error <- dtmp$sw  #error diam
     v.error <- dtmp$sv  #error growth

  if(g < 100)v.error <- muv

     ad[g]   <- dtmp$ad
     ai      <- ai + dtmp$aa
     dtmp    <- diam.t
     dtmp[is.na(surv) | surv != 1] <- NA
     dgrow   <- t(apply(dtmp,1,diff,na.rm=T))   #diameter increment

  if(!COVARIATES)mgibbs[g]  <- mu
  if(COVARIATES) agibbs[g,] <- alpha

  ingibbs[g,] <- beta.i[ins]
  tgibbs[g,]  <- beta.t
  sgibbs[g,]  <- c(sig,sigd,sigp,w.error,v.error)
  if(g > burnin){

    bigibbs     <- bigibbs + beta.i
    bpgibbs     <- bpgibbs + beta.p
    bp2gibbs     <- bp2gibbs + beta.p^2
    dgibbs      <- dgibbs + diam.t
    d2gibbs     <- d2gibbs + diam.t^2
    ggibbs      <- ggibbs + dgrow
    g2gibbs     <- g2gibbs + dgrow^2
    ldgibbs     <- ldgibbs + log(diam.t)
    ld2gibbs    <- ld2gibbs + log(diam.t)^2
    lggibbs     <- lggibbs + log(dgrow)
    lg2gibbs    <- lg2gibbs + log(dgrow)^2
  }
  if(g %in% printseq){
     print(g)
     print(beta.t)
  }
}

save.image(file=paste(spgroup,".output.RData",sep=""))   #write output to file
